/******************************************************************************\
*                                                                              *
*        Copyright (c) 2003, The Regents of the University of California       *
*      See the file COPYRIGHT for a complete copyright notice and license.     *
*                                                                              *
********************************************************************************
*
* CVS info:
*   $RCSfile: parse_options.c,v $
*   $Revision: 1.21 $
*   $Date: 2005/06/29 15:45:09 $
*   $Author: loewe $
*
* Purpose:
*       Parse commandline functions.
*
\******************************************************************************/

#include "IOR.h"
#include "defaults.h"                                 /* IOR defaults */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/******************************************************************************/
/*
 * Check and correct all settings of each test in queue for correctness.
 */

void CheckRunSettings(IOR_queue_t *tests) {
    while (tests != NULL) {
        /* If readFile and writeFile were both left unset, set both now */
        if (tests->testParameters.readFile == tests->testParameters.writeFile) {
            tests->testParameters.readFile = TRUE;
            tests->testParameters.writeFile = TRUE;
        }
        /* If numTasks set to 0, use all tasks */
        if (tests->testParameters.numTasks == 0) {
            MPI_Comm_size(MPI_COMM_WORLD, &tests->testParameters.numTasks);
        }
        tests = tests->nextTest;
    }
} /* CheckRunSettings() */


void DecodeDirective(char *line, IOR_param_t *test)
{
    char option[MAX_STR];
    char value[MAX_STR];
    int rc;

    rc = sscanf(line, " %[^=# \t\r\n] = %[^# \t\r\n] ", option, value);
    if (rc != 2 && rank == 0) {
	fprintf(stdout, "Syntax error in configuration options: %s\n", line);
	MPI_Abort(MPI_COMM_WORLD, 2);
    }
    if (strcasecmp(option, "api") == 0) {
	strcpy(test->api, value);
    } else if (strcasecmp(option, "debug") == 0) {
	strcpy(test->debug, value);
    } else if (strcasecmp(option, "platform") == 0) {
	strcpy(test->platform, value);
    } else if (strcasecmp(option, "testfile") == 0) {
	strcpy(test->testFileName, value);
    } else if (strcasecmp(option, "hintsfilename") == 0) {
	strcpy(test->hintsFileName, value);
    } else if (strcasecmp(option, "maxtimeduration") == 0) {
	test->maxTimeDuration = atoi(value);
    } else if (strcasecmp(option, "nodes") == 0) {
	test->nodes = atoi(value);
    } else if (strcasecmp(option, "repetitions") == 0) {
	test->repetitions = atoi(value);
    } else if (strcasecmp(option, "intertestdelay") == 0) {
	test->interTestDelay = atoi(value);
    } else if (strcasecmp(option, "readfile") == 0) {
	test->readFile = atoi(value);
    } else if (strcasecmp(option, "writefile") == 0) {
	test->writeFile = atoi(value);
    } else if (strcasecmp(option, "fileperproc") == 0) {
	test->filePerProc = atoi(value);
    } else if (strcasecmp(option, "reordertasks") == 0) {
	test->reorderTasks = atoi(value);
    } else if (strcasecmp(option, "checkwrite") == 0) {
	test->checkWrite = atoi(value);
    } else if (strcasecmp(option, "checkread") == 0) {
	test->checkRead = atoi(value);
    } else if (strcasecmp(option, "keepfile") == 0) {
	test->keepFile = atoi(value);
    } else if (strcasecmp(option, "multiFile") == 0) {
	test->multiFile = atoi(value);
    } else if (strcasecmp(option, "quitonerror") == 0) {
	test->quitOnError = atoi(value);
    } else if (strcasecmp(option, "segmentcount") == 0) {
	test->segmentCount = StringToBytes(value);
    } else if (strcasecmp(option, "blocksize") == 0) {
	test->blockSize = StringToBytes(value);
    } else if (strcasecmp(option, "transfersize") == 0) {
	test->transferSize = StringToBytes(value);
    } else if (strcasecmp(option, "singlexferattempt") == 0) {
	test->singleXferAttempt = atoi(value);
    } else if (strcasecmp(option, "individualdatasets") == 0) {
	test->individualDataSets = atoi(value);
    } else if (strcasecmp(option, "intraTestBarriers") == 0) {
	test->intraTestBarriers = atoi(value);
    } else if (strcasecmp(option, "nofill") == 0) {
	test->noFill = atoi(value);
    } else if (strcasecmp(option, "verbose") == 0) {
	test->verbose = atoi(value);
    } else if (strcasecmp(option, "collective") == 0) {
	test->collective = atoi(value);
    } else if (strcasecmp(option, "preallocate") == 0) {
	test->preallocate = atoi(value);
    } else if (strcasecmp(option, "storefileoffset") == 0) {
	test->storeFileOffset = atoi(value);
    } else if (strcasecmp(option, "usefileview") == 0) {
	test->useFileView = atoi(value);
    } else if (strcasecmp(option, "usesharedfilepointer") == 0) {
	test->useSharedFilePointer = atoi(value);
    } else if (strcasecmp(option, "useo_direct") == 0) {
	test->useO_DIRECT = atoi(value);
    } else if (strcasecmp(option, "usestrideddatatype") == 0) {
	test->useStridedDatatype = atoi(value);
    } else if (strcasecmp(option, "showhints") == 0) {
	test->showHints = atoi(value);
    } else if (strcasecmp(option, "showhelp") == 0) {
	test->showHelp = atoi(value);
    } else if (strcasecmp(option, "uniqueDir") == 0) {
	test->uniqueDir = atoi(value);
    } else if (strcasecmp(option, "useexistingtestfile") == 0) {
	test->useExistingTestFile = atoi(value);
    } else if (strcasecmp(option, "fsync") == 0) {
	test->fsync = atoi(value);
    } else if (strcasecmp(option, "lustrestripecount") == 0) {
	test->lustre_stripe_count = atoi(value);
    } else if (strcasecmp(option, "lustrestripesize") == 0) {
	test->lustre_stripe_size = StringToBytes(value);
    } else if (strcasecmp(option, "lustrestartost") == 0) {
	test->lustre_start_ost = atoi(value);
    } else if (strcasecmp(option, "lustreignorelocks") == 0) {
	test->lustre_ignore_locks = atoi(value);
    } else if (strcasecmp(option, "numtasks") == 0) {
	test->numTasks = atoi(value);
    } else {
        ;
	/*
	if (rank == 0)
	    fprintf(stdout, "Unrecognized parameter \"%s\"\n", option);
	MPI_Abort(MPI_COMM_WORLD, 1);
	*/
    }
} /* DecodeDirective() */


/******************************************************************************/
/*
 * Parse a single line, which may contain multiple comma-seperated directives
 */

void
ParseLine(char *line, IOR_param_t *test)
{
    char *start, *end;

    start = line;
    do {
	end = strchr(start, ',');
	if (end != NULL)
	    *end = '\0';
	DecodeDirective(start, test);
	start = end + 1;
    } while (end != NULL);

} /* ParseLine() */


/******************************************************************************/
/*
 * Determines if the string "haystack" contains only the string "needle", and
 * possibly whitespace before and after needle.  Function is case insensitive.
 */
int
contains_only(char *haystack, char *needle)
{
    char *ptr, *end;

    end = haystack + strlen(haystack);
    /* skip over leading shitspace */
    for (ptr = haystack; ptr < end; ptr++) {
	if (!isspace(*ptr))
	    break;
    }
    /* check for "needle" */
    if (strncasecmp(ptr, needle, strlen(needle)) != 0)
	return 0;
    /* make sure the rest of the line is only whitspace as well */
    for (ptr += strlen(needle); ptr < end; ptr++) {
	if (!isspace(*ptr))
	    return 0;
    }

    return 1;
} /* contains_only() */


/******************************************************************************/
/*
 * Read the configuration script, allocating and filling in the structure of
 * global parameters.
 */

IOR_queue_t *
ReadConfigScript(char * scriptName)
{
    int          test_num = 0;
    int          runflag = 0;
    char         linebuf[MAX_STR];
    char         empty[MAX_STR];
    FILE        *file;
    IOR_queue_t *head    = NULL;
    IOR_queue_t *tail    = NULL;
    IOR_queue_t *newTest = NULL;

    /* Initialize the first test */
    head = CreateNewTest(test_num++);
    tail = head;

    /* open the script */
    file = fopen(scriptName, "r");
    if (file == NULL)
	ERR("cannot open file");

    /* search for the "IOR START" line */
    while(fgets(linebuf, MAX_STR, file) != NULL) {
	if (contains_only(linebuf, "ior start")) {
	    break;
	}
    }

    /* Iterate over a block of IOR commands */
    while(fgets(linebuf, MAX_STR, file) != NULL) {
	/* skip empty lines */
	if (sscanf(linebuf, "%s", empty) == -1)
	    continue;
	/* skip lines containing only comments */
	if (sscanf(linebuf, " #%s", empty) == 1)
	    continue;
	if (contains_only(linebuf, "ior stop")) {
	    break;
	} else if (contains_only(linebuf, "run")) {
	    runflag = 1;
	} else {
            /* If this directive was preceded by a "run" line, then
	       create and initialize a new test structure */
	    if (runflag) {
		newTest = (IOR_queue_t *)malloc(sizeof(IOR_queue_t));
		if (newTest == NULL)
		    ERR("malloc failed");
		newTest->testParameters = tail->testParameters;
		newTest->testParameters.id = test_num++;
		tail->nextTest = newTest;
		tail = newTest;
		tail->nextTest = NULL;
		runflag = 0;
	    }
	    ParseLine(linebuf, &tail->testParameters);
	}
    }

    /* close the script */
    if (fclose(file) != 0)
	ERR("cannot close script file");

    return(head);
} /* ReadConfigScript() */


/******************************************************************************/
/*
 * Parse Commandline.
 */

IOR_queue_t *
ParseCommandLine(int argc, char ** argv)
{
    static const char * opts  = "a:b:BcCd:eEf:FghHi:IklmnN:o:O:pPqrRs:St:T:uU:vVwWx";
    int                 c, i;
    static IOR_queue_t *tests = NULL;

    /* suppress getopt() error message when a character is unrecognized */
    opterr = 0;

    defaultParameters.readFile = defaultParameters.writeFile = FALSE;

    while ((c = getopt(argc, argv, opts)) != -1) {
        switch (c) {
	case 'a': strcpy(defaultParameters.api, optarg);           break;
	case 'b': defaultParameters.blockSize =
		      StringToBytes(optarg);                       break;
	case 'B': defaultParameters.useO_DIRECT = TRUE;            break;
	case 'c': defaultParameters.collective = TRUE;             break;
	case 'C': defaultParameters.reorderTasks = TRUE;           break;
	case 'd': defaultParameters.interTestDelay = atoi(optarg); break;
	case 'e': defaultParameters.fsync = TRUE;                  break;
	case 'E': defaultParameters.useExistingTestFile = TRUE;    break;
	case 'f': tests = ReadConfigScript(optarg);                break;
	case 'F': defaultParameters.filePerProc = TRUE;            break;
	case 'g': defaultParameters.intraTestBarriers = TRUE;      break;
	case 'h': defaultParameters.showHelp = TRUE;               break;
	case 'H': defaultParameters.showHints = TRUE;              break;
	case 'i': defaultParameters.repetitions = atoi(optarg);    break;
	case 'I': defaultParameters.individualDataSets = TRUE;     break;
	case 'k': defaultParameters.keepFile = TRUE;               break;
	case 'l': defaultParameters.storeFileOffset = TRUE;        break;
	case 'm': defaultParameters.multiFile = TRUE;              break;
	case 'n': defaultParameters.noFill = TRUE;                 break;
	case 'N': defaultParameters.numTasks = atoi(optarg);       break;
	case 'o': strcpy(defaultParameters.testFileName, optarg);  break;
	case 'O': ParseLine(optarg, &defaultParameters);           break;
	case 'p': defaultParameters.preallocate = TRUE;            break;
	case 'P': defaultParameters.useSharedFilePointer = TRUE;   break;
	case 'q': defaultParameters.quitOnError = TRUE;            break;
	case 'r': defaultParameters.readFile = TRUE;               break;
	case 'R': defaultParameters.checkRead = TRUE;              break;
	case 's': defaultParameters.segmentCount = atoi(optarg);   break;
	case 'S': defaultParameters.useStridedDatatype = TRUE;     break;
	case 't': defaultParameters.transferSize =
		      StringToBytes(optarg);                       break;
	case 'T': defaultParameters.maxTimeDuration = atoi(optarg);break;
	case 'u': defaultParameters.uniqueDir = TRUE;              break;
	case 'U': strcpy(defaultParameters.hintsFileName, optarg); break;
	case 'v': defaultParameters.verbose++;                     break;
	case 'V': defaultParameters.useFileView = TRUE;            break;
	case 'w': defaultParameters.writeFile = TRUE;              break;
	case 'W': defaultParameters.checkWrite = TRUE;             break;
	case 'x': defaultParameters.singleXferAttempt = TRUE;      break;
	default: fprintf (stderr, "unknown option `-%c'.\n", optopt);
        }
    }

    for (i = optind; i < argc; i++)
        fprintf (stdout, "non-option argument: %s\n", argv[i]);

    /* If an IOR script was not used, initialize test queue to the defaults */
    if (tests == NULL) {
	tests = (IOR_queue_t *) malloc (sizeof(IOR_queue_t));
	if (!tests)
	    ERR("malloc failed");
	tests->testParameters = defaultParameters;
	tests->nextTest = NULL;
    }

    CheckRunSettings(tests);

    return(tests);
} /* ParseCommandLine() */
